!MNH_LIC Copyright 2016-2019 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
! Original version:
!  Philippe Wautelet: 05/2016-04/2018: new data structures and calls for I/O
! Modifications:
!  Philippe Wautelet: 29/01/2019 : small bug correction (null pointers) in FIELDLIST_GOTO_MODEL if NESPGD or PGD
!  Philippe Wautelet: 01/02/2019 : bug correction in case XRT is not associated
!-----------------------------------------------------------------
MODULE MODE_FIELD
!
USE MODD_CONF,      ONLY : CPROGRAM
USE MODD_IO_ll,     ONLY : NVERB_DEBUG,NVERB_INFO,NVERB_WARNING,NVERB_ERROR,NVERB_FATAL
USE MODD_PARAMETERS
USE MODD_TYPE_DATE, ONLY : DATE_TIME
#if defined(MNH_IOCDF4)
USE NETCDF,         ONLY : NF90_FILL_INT, NF90_FILL_REAL
#endif
!
USE MODE_MSG
!
IMPLICIT NONE
!
INTEGER,PRIVATE,PARAMETER :: MAXFIELDS = 250
INTEGER,PARAMETER :: TYPEUNDEF = -1, TYPEINT = 1, TYPELOG = 2, TYPEREAL = 3, TYPECHAR = 4, TYPEDATE = 5
!
TYPE TFIELDPTR_C0D
  CHARACTER(LEN=:),     POINTER :: DATA => NULL()
END TYPE TFIELDPTR_C0D
!
TYPE TFIELDPTR_L0D
  LOGICAL,              POINTER :: DATA => NULL()
END TYPE TFIELDPTR_L0D
!
TYPE TFIELDPTR_N0D
  INTEGER,              POINTER :: DATA => NULL()
END TYPE TFIELDPTR_N0D
!
TYPE TFIELDPTR_N2D
  INTEGER,DIMENSION(:,:), POINTER :: DATA => NULL()
END TYPE TFIELDPTR_N2D
!
TYPE TFIELDPTR_N3D
  INTEGER,DIMENSION(:,:,:),POINTER :: DATA => NULL()
END TYPE TFIELDPTR_N3D
!
TYPE TFIELDPTR_X0D
  REAL,                 POINTER :: DATA => NULL()
END TYPE TFIELDPTR_X0D
!
TYPE TFIELDPTR_X1D
  REAL,DIMENSION(:),    POINTER :: DATA => NULL()
END TYPE TFIELDPTR_X1D
!
TYPE TFIELDPTR_X2D
  REAL,DIMENSION(:,:),  POINTER :: DATA => NULL()
END TYPE TFIELDPTR_X2D
!
TYPE TFIELDPTR_X3D
  REAL,DIMENSION(:,:,:),POINTER :: DATA => NULL()
END TYPE TFIELDPTR_X3D
!
TYPE TFIELDPTR_X4D
  REAL,DIMENSION(:,:,:,:),POINTER :: DATA => NULL()
END TYPE TFIELDPTR_X4D
!
TYPE TFIELDPTR_T0D
  TYPE(DATE_TIME),      POINTER :: DATA => NULL()
END TYPE TFIELDPTR_T0D
!
!Structure describing the characteristics of a field
TYPE TFIELDDATA
  CHARACTER(LEN=NMNHNAMELGTMAX) :: CMNHNAME  = '' !Name of the field (for MesoNH, non CF convention)
  CHARACTER(LEN=NSTDNAMELGTMAX) :: CSTDNAME  = '' !Standard name (CF convention)
  CHARACTER(LEN=32)  :: CLONGNAME = '' !Long name (CF convention)
  CHARACTER(LEN=40)  :: CUNITS    = '' !Canonical units (CF convention)
  CHARACTER(LEN=2)   :: CDIR      = '' !Type of the data field (XX,XY,--...)
  CHARACTER(LEN=4)   :: CLBTYPE   = 'NONE' !Type of the lateral boundary (LBX,LBY,LBXU,LBYV)
  CHARACTER(LEN=100) :: CCOMMENT  = '' !Comment (for MesoNH, non CF convention)
  INTEGER            :: NGRID     = NGRIDUNKNOWN !Localization on the model grid
  INTEGER            :: NTYPE     = TYPEUNDEF !Datatype
  INTEGER            :: NDIMS     = 0  !Number of dimensions
  LOGICAL            :: LTIMEDEP  = .FALSE. !Is the field time-dependent?
  !
#if defined(MNH_IOCDF4)
  INTEGER            :: NFILLVALUE =  NF90_FILL_INT  !Fill value for integer fields
  REAL               :: XFILLVALUE =  NF90_FILL_REAL !Fill value for real fields
                                                     !NF90_FILL_REAL is the default fill value
                                                     !used by netCDF to pre-fill real and also double
                                                     !variables
#else
  INTEGER            :: NFILLVALUE =  -2147483647            !Fill value for integer fields
  REAL               :: XFILLVALUE =  9.9692099683868690e+36 !Fill value for real fields
#endif
  INTEGER            :: NVALIDMIN  = -2147483646 !Minimum valid value for integer fields
  INTEGER            :: NVALIDMAX  =  2147483647 !Maximum valid value for integer fields
  REAL               :: XVALIDMIN  = -1.E36 !Minimum valid value for real fields
  REAL               :: XVALIDMAX  =  1.E36 !Maximum valid value for real fields
  !
  TYPE(TFIELDPTR_C0D),DIMENSION(:),ALLOCATABLE :: TFIELD_C0D !Pointer to the character string fields (one per nested mesh)
  !
  TYPE(TFIELDPTR_L0D),DIMENSION(:),ALLOCATABLE :: TFIELD_L0D !Pointer to the scalar logical fields (one per nested mesh)
  !
  TYPE(TFIELDPTR_N0D),DIMENSION(:),ALLOCATABLE :: TFIELD_N0D !Pointer to the scalar integer fields (one per nested mesh)
  TYPE(TFIELDPTR_N2D),DIMENSION(:),ALLOCATABLE :: TFIELD_N2D !Pointer to the integer 2D fields (one per nested mesh)
  TYPE(TFIELDPTR_N3D),DIMENSION(:),ALLOCATABLE :: TFIELD_N3D !Pointer to the integer 3D fields (one per nested mesh)
  !
  TYPE(TFIELDPTR_X0D),DIMENSION(:),ALLOCATABLE :: TFIELD_X0D !Pointer to the scalar real fields (one per nested mesh)
  TYPE(TFIELDPTR_X1D),DIMENSION(:),ALLOCATABLE :: TFIELD_X1D !Pointer to the real 1D fields (one per nested mesh)
  TYPE(TFIELDPTR_X2D),DIMENSION(:),ALLOCATABLE :: TFIELD_X2D !Pointer to the real 2D fields (one per nested mesh)
  TYPE(TFIELDPTR_X3D),DIMENSION(:),ALLOCATABLE :: TFIELD_X3D !Pointer to the real 3D fields (one per nested mesh)
  TYPE(TFIELDPTR_X4D),DIMENSION(:),ALLOCATABLE :: TFIELD_X4D !Pointer to the real 4D fields (one per nested mesh)
  !
  TYPE(TFIELDPTR_T0D),DIMENSION(:),ALLOCATABLE :: TFIELD_T0D !Pointer to the scalar date/time fields (one per nested mesh)
END TYPE TFIELDDATA
!
LOGICAL,SAVE :: LFIELDLIST_ISINIT = .FALSE.
TYPE(TFIELDDATA),DIMENSION(MAXFIELDS),SAVE :: TFIELDLIST
!
CONTAINS
!
SUBROUTINE INI_FIELD_LIST(KMODEL)
! Modif
!  J.Escobar 25/04/2018: missing def of FRC
!------------------------------------------------
USE MODD_CONF, ONLY: NMODEL
!
INTEGER,INTENT(IN),OPTIONAL :: KMODEL
!
INTEGER :: IDX, IMODEL
CHARACTER(LEN=42) :: YMSG
!
!F90/95: TFIELDLIST(1) = TFIELDDATA('UT','x_wind','m s-1','XY','X_Y_Z_U component of wind',2)
!F2003:
!TFIELDLIST(1) = TFIELDDATA(CMNHNAME='UT',CSTDNAME='x_wind',CUNITS='m s-1',CDIR='XY',&
!                           CCOMMENT='X_Y_Z_U component of wind',NGRID=2)
!
CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_LIST','called')
IF (LFIELDLIST_ISINIT) THEN
  CALL PRINT_MSG(NVERB_ERROR,'GEN','INI_FIELD_LIST','already called')
  RETURN
END IF
LFIELDLIST_ISINIT = .TRUE.
!
IF (PRESENT(KMODEL)) THEN
  IMODEL = KMODEL
ELSE
  IF (NMODEL/=1) THEN
    IMODEL = NMODEL
  ELSE !NMODEL is not necessary known here => allocating for max allowed number of models
    IMODEL = JPMODELMAX
  END IF
END IF
IF (IMODEL==0) CALL PRINT_MSG(NVERB_FATAL,'GEN','INI_FIELD_LIST','allocating fields for zero models not allowed')
!
WRITE(YMSG,'("allocating fields for up to ",I4," model(s)")') IMODEL
CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_LIST',YMSG)
!
IDX = 1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MNHVERSION'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MesoNH version'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MASDEV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MesoNH version (without bugfix)'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'BUGFIX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MesoNH bugfix number'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'BIBUSER'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MesoNH: user binary library'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VERSION'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SURFEX version (without BUG)'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'BUG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SURFEX bugfix number'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PROGRAM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MesoNH family: used program'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'FILETYPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'type of this file'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MY_NAME'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'filename (no extension)'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DAD_NAME'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'filename of the dad file'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DXRATIO'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DXRATIO'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Resolution ratio between this mesh and its father in x-direction'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DYRATIO'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DYRATIO'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Resolution ratio between this mesh and its father in y-direction'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'XSIZE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'XSIZE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of model 1 grid points in x-direction in the model 2 physical domain'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'YSIZE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'YSIZE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of model 1 grid points in y-direction in the model 2 physical domain'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'XOR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'XOR'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Horizontal position of this mesh relative to its father'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'YOR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'YOR'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Vertical position of this mesh relative to its father'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'STORAGE_TYPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'STORAGE_TYPE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Storage type for the information written in the FM files'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'IMAX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'IMAX'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'x-dimension of the physical domain'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'JMAX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'JMAX'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'y-dimension of the physical domain'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'KMAX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'KMAX'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'z-dimension of the physical domain'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'JPHEXT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'JPHEXT'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of horizontal external points on each side'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RPK'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RPK'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Projection parameter for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LONORI'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LONORI'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Longitude of the point of coordinates x=0, y=0 for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LATORI'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LATORI'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Latitude of the point of coordinates x=0, y=0 for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LONOR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LONOR'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Longitude of 1st mass point'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LATOR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LATOR'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Latitude of 1st mass point'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'THINSHELL'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'THINSHELL'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for thinshell approximation'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LAT0'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LAT0'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Reference latitude for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LON0'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LON0'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Reference longitude for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'BETA'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'BETA'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Rotation angle for conformal projection'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'XHAT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = 'projection_x_coordinate'
TFIELDLIST(IDX)%CLONGNAME  = 'XHAT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XX'
TFIELDLIST(IDX)%CCOMMENT   = 'Position x in the conformal or cartesian plane'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'YHAT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = 'projection_y_coordinate'
TFIELDLIST(IDX)%CLONGNAME  = 'YHAT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'YY'
TFIELDLIST(IDX)%CCOMMENT   = 'Position y in the conformal or cartesian plane'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ZHAT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ZHAT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'ZZ'
TFIELDLIST(IDX)%CCOMMENT   = 'Height level without orography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ZTOP'
TFIELDLIST(IDX)%CSTDNAME   = 'altitude_at_top_of_atmosphere_model'
TFIELDLIST(IDX)%CLONGNAME  = 'ZTOP'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Height of top level'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DXHAT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DXHAT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XX'
TFIELDLIST(IDX)%CCOMMENT   = 'Horizontal stretching in x'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DYHAT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DYHAT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'YY'
TFIELDLIST(IDX)%CCOMMENT   = 'Horizontal stretching in y'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ALT'
TFIELDLIST(IDX)%CSTDNAME   = 'altitude'
TFIELDLIST(IDX)%CLONGNAME  = 'ALT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_ALTitude'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIRCOSXW'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIRCOSXW'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X director cosinus of the normal to the ground surface'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIRCOSYW'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIRCOSYW'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Y director cosinus of the normal to the ground surface'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIRCOSZW'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIRCOSZW'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Z director cosinus of the normal to the ground surface'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'COSSLOPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'COSSLOPE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'cosinus of the angle between i and the slope vector'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SINSLOPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SINSLOPE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'sinus of the angle between i and the slope vector'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MAP'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MAP'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Map factor'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'latitude'
TFIELDLIST(IDX)%CSTDNAME   = 'latitude'
TFIELDLIST(IDX)%CLONGNAME  = 'latitude'
TFIELDLIST(IDX)%CUNITS     = 'degrees_north'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_latitude at mass point'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'longitude'
TFIELDLIST(IDX)%CSTDNAME   = 'longitude'
TFIELDLIST(IDX)%CLONGNAME  = 'longitude'
TFIELDLIST(IDX)%CUNITS     = 'degrees_east'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_longitude at mass point'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'latitude_u'
TFIELDLIST(IDX)%CSTDNAME   = 'latitude_at_u_location'
TFIELDLIST(IDX)%CLONGNAME  = 'latitude at u location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_north'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_latitude at u point'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'longitude_u'
TFIELDLIST(IDX)%CSTDNAME   = 'longitude_at_u_location'
TFIELDLIST(IDX)%CLONGNAME  = 'longitude at u location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_east'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_longitude at u point'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'latitude_v'
TFIELDLIST(IDX)%CSTDNAME   = 'latitude_at_v_location'
TFIELDLIST(IDX)%CLONGNAME  = 'latitude at v location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_north'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_latitude at v point'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'longitude_v'
TFIELDLIST(IDX)%CSTDNAME   = 'longitude_at_v_location'
TFIELDLIST(IDX)%CLONGNAME  = 'longitude at v location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_east'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_longitude at v point'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'latitude_f'
TFIELDLIST(IDX)%CSTDNAME   = 'latitude_at_f_location'
TFIELDLIST(IDX)%CLONGNAME  = 'latitude at f location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_north'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_latitude at f point'
TFIELDLIST(IDX)%NGRID      = 5
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'longitude_f'
TFIELDLIST(IDX)%CSTDNAME   = 'longitude_at_f_location'
TFIELDLIST(IDX)%CLONGNAME  = 'longitude at f location'
TFIELDLIST(IDX)%CUNITS     = 'degrees_east'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_longitude at f point'
TFIELDLIST(IDX)%NGRID      = 5
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LAT'
! TFIELDLIST(IDX)%CSTDNAME   = 'latitude'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LAT'
TFIELDLIST(IDX)%CUNITS     = 'degrees_north'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_latitude'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LON'
! TFIELDLIST(IDX)%CSTDNAME   = 'longitude'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LON'
TFIELDLIST(IDX)%CUNITS     = 'degrees_east'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_longitude'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ZS'
TFIELDLIST(IDX)%CSTDNAME   = 'surface_altitude'
TFIELDLIST(IDX)%CLONGNAME  = 'ZS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'orography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ZSMT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ZSMT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'smooth orography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SLEVE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SLEVE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for SLEVE coordinate'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_L0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LEN1'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LEN1'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Decay scale for smooth topography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LEN2'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LEN2'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Decay scale for small-scale topography deviation'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTMOD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTMOD'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of model beginning'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_T0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTCUR'
TFIELDLIST(IDX)%CSTDNAME   = 'time'
TFIELDLIST(IDX)%CLONGNAME  = 'DTCUR'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Current time and date'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_T0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTRAD_FULL'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTRAD_FULL'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of last full radiation call'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_T0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTRAD_CLLY'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTRAD_CLLY'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of last radiation call for only cloudy verticals'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_T0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTDCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTDCONV'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of the last deep convection call'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_T0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTEXP'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTEXP'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of experiment beginning'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTSEG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTSEG'
TFIELDLIST(IDX)%CUNITS     = 'seconds since YYYY-MM-DD HH:MM:SS.S'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Time and date of segment beginning'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEDATE
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'L1D'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'L1D'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for 1D model version'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'L2D'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'L2D'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for 2D model version'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PACK'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PACK'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical to compress 1D or 2D FM files'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CARTESIAN'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CARTESIAN'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for cartesian geometry'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBOUSS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBOUSS'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for Boussinesq approximation'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SURF'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SURF'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Kind of surface processes parameterization'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPECHAR
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_C0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CPL_AROME'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CPL_AROME'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for AROME coupling file'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'COUPLING'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'COUPLING'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Logical for coupling file'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UT'
TFIELDLIST(IDX)%CSTDNAME   = 'x_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'UT'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VT'
TFIELDLIST(IDX)%CSTDNAME   = 'y_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'VT'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'WT'
TFIELDLIST(IDX)%CSTDNAME   = 'upward_air_velocity'
TFIELDLIST(IDX)%CLONGNAME  = 'WT'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_vertical wind'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'THT'
TFIELDLIST(IDX)%CSTDNAME   = 'air_potential_temperature'
TFIELDLIST(IDX)%CLONGNAME  = 'THT'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_potential temperature'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UM'
TFIELDLIST(IDX)%CSTDNAME   = 'x_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'UM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VM'
TFIELDLIST(IDX)%CSTDNAME   = 'y_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'VM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'WM'
TFIELDLIST(IDX)%CSTDNAME   = 'upward_air_velocity'
TFIELDLIST(IDX)%CLONGNAME  = 'WM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_vertical wind'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DUM'
TFIELDLIST(IDX)%CSTDNAME   = 'x_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'DUM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DVM'
TFIELDLIST(IDX)%CSTDNAME   = 'y_wind'
TFIELDLIST(IDX)%CLONGNAME  = 'DVM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DWM'
TFIELDLIST(IDX)%CSTDNAME   = 'upward_air_velocity'
TFIELDLIST(IDX)%CLONGNAME  = 'DWM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_vertical wind'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TKET'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TKET'
TFIELDLIST(IDX)%CUNITS     = 'm2 s-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Turbulent Kinetic Energy'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TKEMS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TKEMS'
TFIELDLIST(IDX)%CUNITS     = 'm2 s-3'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Turbulent Kinetic Energy adv source'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PABST'
TFIELDLIST(IDX)%CSTDNAME   = 'air_pressure'
TFIELDLIST(IDX)%CLONGNAME  = 'PABST'
TFIELDLIST(IDX)%CUNITS     = 'Pa'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_ABSolute Pressure'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Moist variables (rho Rn)'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 4
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X4D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RVT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = 'humidity_mixing_ratio'
TFIELDLIST(IDX)%CLONGNAME  = 'RVT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Vapor mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RCT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RCT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Cloud mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RRT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RRT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Rain mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RIT'
!TODO: check stdname
TFIELDLIST(IDX)%CSTDNAME   = 'cloud_ice_mixing_ratio'
TFIELDLIST(IDX)%CLONGNAME  = 'RIT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Ice mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RST'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RST'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Snow mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RGT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RGT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Graupel mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RHT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RHT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Hail mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
TFIELDLIST(IDX)%NDIMS      = 3
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SUPSATMAX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SUPSATMAX'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Supersaturation'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'NACT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'NACT'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Nact'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SSPRO'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SSPRO'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Supersaturation'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'NPRO'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'NPRO'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Nact'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPAP'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPAP'
TFIELDLIST(IDX)%CUNITS     = 'kg m-2 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous Precipitating Aerosol Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPAP'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPAP'
TFIELDLIST(IDX)%CUNITS     = 'kg m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated Precipitating Aerosol Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EFIELDU'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'EFIELDU'
TFIELDLIST(IDX)%CUNITS     = 'V m-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_EFIELDU'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EFIELDV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'EFIELDV'
TFIELDLIST(IDX)%CUNITS     = 'V m-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_EFIELDV'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EFIELDW'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'EFIELDW'
TFIELDLIST(IDX)%CUNITS     = 'V m-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_EFIELDW'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'NI_IAGGS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'NI_IAGGS'
TFIELDLIST(IDX)%CUNITS     = 'C m-3 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_NI_IAGGS'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'NI_IDRYG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'NI_IDRYG'
TFIELDLIST(IDX)%CUNITS     = 'C m-3 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_NI_IDRYG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'NI_SDRYG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'NI_SDRYG'
TFIELDLIST(IDX)%CUNITS     = 'C m-3 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_NI_SDRYG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INDUC_CG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INDUC_CG'
TFIELDLIST(IDX)%CUNITS     = 'C m-3 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_INDUC_CG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TRIG_IC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TRIG_IC'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_FLASH_MAP_TRIG_IC'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'IMPACT_CG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'IMPACT_CG'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_FLASH_MAP_IMPACT_CG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'AREA_CG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'AREA_CG'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_FLASH_MAP_2DAREA_CG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'AREA_IC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'AREA_IC'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_FLASH_MAP_2DAREA_IC'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'FLASH_3DCG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'FLASH_3DCG'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_FLASH_MAP_3DCG'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'FLASH_3DIC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'FLASH_3DIC'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_FLASH_MAP_3DIC'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PHC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PHC'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'pH in cloud'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PHR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PHR'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'pH in rain'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LSUM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LSUM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Large Scale U component'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LSVM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LSVM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Large Scale V component'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LSWM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LSWM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Large Scale vertical wind'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LSTHM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LSTHM'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Large Scale potential Temperature'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LSRVM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LSRVM'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Large Scale Vapor Mixing Ratio'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RIMX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RIMX'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of points in the lateral absorbing layer in the x direction'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RIMY'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RIMY'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of points in the lateral absorbing layer in the y direction'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'HORELAX_UVWTH'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'HORELAX_UVWTH'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Switch to activate the HOrizontal RELAXation'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_L0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBXUM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBXUM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBXU'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBXUM'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBXVM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBXVM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBX'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBXVM'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBXWM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBXWM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBX'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBXWM'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBYUM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBYUM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBY'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBYUM'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBYVM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBYVM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBYV'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBYVM'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBYWM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBYWM'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBY'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBYWM'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBXTHM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBXTHM'
TFIELDLIST(IDX)%CUNITS     = 'K'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBX'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBXTHM'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBYTHM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBYTHM'
TFIELDLIST(IDX)%CUNITS     = 'K'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBY'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBYTHM'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'HORELAX_TKE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'HORELAX_TKE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Switch to activate the HOrizontal RELAXation'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPELOG
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_L0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBXTKEM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBXTKEM'
TFIELDLIST(IDX)%CUNITS     = 'm2 s-2'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBX'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBXTKEM'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'LBYTKEM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'LBYTKEM'
TFIELDLIST(IDX)%CUNITS     = 'm2 s-2'
! TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = 'LBY'
TFIELDLIST(IDX)%CCOMMENT   = '2_Y_Z_LBYTKEM'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DRYMASST'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DRYMASST'
TFIELDLIST(IDX)%CUNITS     = 'kg'
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Total Dry Mass'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X0D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'BL_DEPTH'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'BL_DEPTH'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_BL_DEPTH'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SBL_DEPTH'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SBL_DEPTH'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_BL_SDEPTH'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'WTHVMF'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'WTHVMF'
TFIELDLIST(IDX)%CUNITS     = 'm K s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_WTHVMF'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SRCT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SRCT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_normalized 2nd_order moment s_r_c/2Sigma_s2'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SIGS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SIGS'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Sigma_s from turbulence scheme'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RHOREFZ'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RHOREFZ'
TFIELDLIST(IDX)%CUNITS     = 'kg m-3'
TFIELDLIST(IDX)%CDIR       = 'ZZ'
TFIELDLIST(IDX)%CCOMMENT   = 'rhodz for reference state without orography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'THVREFZ'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'THVREFZ'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'ZZ'
TFIELDLIST(IDX)%CCOMMENT   = 'thetavz for reference state without orography'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 1
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X1D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EXNTOP'
TFIELDLIST(IDX)%CSTDNAME   = 'dimensionless_exner_function'
TFIELDLIST(IDX)%CLONGNAME  = 'EXNTOP'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Exner function at model top'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
!
IF (TRIM(CPROGRAM) == 'MESONH' .OR. TRIM(CPROGRAM) == 'DIAG' .OR. TRIM(CPROGRAM) == 'LFICDF') THEN
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'US_PRES'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'US_PRES'
!TODO: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_US_PRES'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VS_PRES'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VS_PRES'
!TODO: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_VS_PRES'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'WS_PRES'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'WS_PRES'
!TODO: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_WS_PRES'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'THS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'THS_CLD'
!TODO: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_THS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Source of Moist variables'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 4
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X4D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RVS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RVS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RVS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RCS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RCS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RCS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RRS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RRS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RRS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RIS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RIS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RIS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RSS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RSS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RSS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RGS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RGS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RGS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RHS_CLD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RHS_CLD'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RHS_CLD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CLDFR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CLDFR'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_CLouD FRaction'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CIT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CIT'
TFIELDLIST(IDX)%CUNITS     = 'm-3'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Cloud Ice concentration'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
END IF ! CPROGRAM=MESONH .OR. DIAG .OR. LFICDF
!
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RHODREF'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RHODREF'
TFIELDLIST(IDX)%CUNITS     = 'kg m-3'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Dry density for reference state with orography'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'THVREF'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'THVREF'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Thetav for reference state with orography'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
!
IF (     TRIM(CPROGRAM) =='MESONH' .OR. TRIM(CPROGRAM) == 'DIAG'  &
    .OR. TRIM(CPROGRAM) == 'LFICDF'.OR. TRIM(CPROGRAM) == 'SPAWN' ) THEN
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTHRAD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTHRAD'
TFIELDLIST(IDX)%CUNITS     = 'K s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_RADiative heating/cooling rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'FLALWD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'FLALWD'
TFIELDLIST(IDX)%CUNITS     = 'W m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Downward Long Waves on FLAT surface'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIRFLASWD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIRFLASWD'
TFIELDLIST(IDX)%CUNITS     = 'W m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_DIRect Downward Short Waves on FLAT surface'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SCAFLASWD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SCAFLASWD'
TFIELDLIST(IDX)%CUNITS     = 'W m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SCAttered Downward Short Waves on FLAT surface'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIRSRFSWD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIRSRFSWD'
TFIELDLIST(IDX)%CUNITS     = 'W m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_DIRect Downward Short Waves'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CLEARCOL_TM1'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CLEARCOL_TM1'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'TRACE OF CLOUD'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ZENITH'
TFIELDLIST(IDX)%CSTDNAME   = 'zenith_angle'
TFIELDLIST(IDX)%CLONGNAME  = 'ZENITH'
TFIELDLIST(IDX)%CUNITS     = 'rad'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ZENITH'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'AZIM'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'AZIM'
TFIELDLIST(IDX)%CUNITS     = 'rad'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_AZIMuth'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DIR_ALB'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DIR_ALB'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_DIRect ALBedo'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SCA_ALB'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SCA_ALB'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SCAttered ALBedo'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EMIS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'EMIS'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_EMISsivity'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TSRAD'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TSRAD'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_RADiative Surface Temperature'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
END IF !CPROGRAM=MESONH .OR. DIAG .OR. LFICDF .OR. SPAWN
!
!
IF ( TRIM(CPROGRAM) /= 'PGD' .AND. TRIM(CPROGRAM) /= 'NESPGD' ) THEN
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'COUNTCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'COUNTCONV'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_COUNTCONV'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DTHCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DTHCONV'
TFIELDLIST(IDX)%CUNITS     = 'K s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_CONVective heating/cooling rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DRVCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DRVCONV'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_CONVective R_v tendency'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DRCCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DRCCONV'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_CONVective R_c tendency'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DRICONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DRICONV'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_CONVective R_i tendency'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRCONV'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_CONVective instantaneous Precipitation Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PACCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PACCONV'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_CONVective ACcumulated Precipitation rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRSCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRSCONV'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_CONVective instantaneous Precipitation Rate for Snow'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DSVCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DSVCONV'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Tracer tendencies'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 4
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X4D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRLFLXCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRLFLXCONV'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Liquid Precipitation Convective Flux'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRSFLXCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRSFLXCONV'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Solid Precipitation Convective Flux'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UMFCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'UMFCONV'
TFIELDLIST(IDX)%CUNITS     = 'kg s-1 m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Updraft Convective Mass Flux'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'DMFCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'DMFCONV'
TFIELDLIST(IDX)%CUNITS     = 'kg s-1 m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Downdraft Convective Mass Flux'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MFCONV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MFCONV'
TFIELDLIST(IDX)%CUNITS     = 'kg s-1 m-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Convective Mass Flux'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CAPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CAPE'
TFIELDLIST(IDX)%CUNITS     = 'J kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Convective Available Potentiel Energy'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CLTOPCONV_LVL'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CLTOPCONV_LVL'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Convective cloud top level'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CLBASCONV_LVL'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CLBASCONV_LVL'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'Convective cloud base level'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_N2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'IC_RATE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'IC_RATE'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_IntraCloud lightning Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CG_RATE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CG_RATE'
TFIELDLIST(IDX)%CUNITS     = 's-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_CloudGround lightning Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'IC_TOTAL_NB'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'IC_TOTAL_NB'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_IntraCloud lightning Number'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'CG_TOTAL_NB'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'CG_TOTAL_NB'
TFIELDLIST(IDX)%CUNITS     = '1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_CloudGround lightning Number'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
END IF !CPROGRAM/=PGD , NESPGD
!
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SSO_ANIS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SSO_ANIS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SSO_ANISOTROPY'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SSO_SLOPE'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SSO_SLOPE'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SSO_SLOPE'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SSO_DIR'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SSO_DIR'
TFIELDLIST(IDX)%CUNITS     = 'degree'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SSO_DIR'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'AVG_ZS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'AVG_ZS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_AVG_ZS'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SIL_ZS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SIL_ZS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SIL_ZS'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MAX_ZS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MAX_ZS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_MAX_ZS'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'MIN_ZS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'MIN_ZS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_MIN_ZS'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'SSO_STDEV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'SSO_STDEV'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_SSO_STDEV'
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRC'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous Cloud Precipitation Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRC'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated Cloud Precipitation Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INDEP'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INDEP'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous Cloud Deposition Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACDEP'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACDEP'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated Cloud Deposition Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRR'
TFIELDLIST(IDX)%CSTDNAME   = 'rainfall_rate'
TFIELDLIST(IDX)%CLONGNAME  = 'INPRR'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous Precipitation Rain Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRR3D'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRR3D'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous 3D Rain Precipitation flux'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'EVAP3D'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'EVAP3D'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1 s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous 3D Rain Evaporation flux'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRR'
TFIELDLIST(IDX)%CSTDNAME   = 'rainfall_amount'
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRR'
!PW: TODO: CF-convention prefers 'kg m-2'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated Precipitation Rain Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRS'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous PRecipitation Snow Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRS'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated PRecipitation Snow Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRG'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous PRecipitation Graupel Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRG'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRG'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated PRecipitation Graupel Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRH'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRH'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_INstantaneous PRecipitation Hail Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRH'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRH'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_ACcumulated PRecipitation Hail Rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'INPRT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'INPRT'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Total INstantaneaous PRecipitation rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
!No permanent variable associated to this field
!ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'ACPRT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'ACPRT'
TFIELDLIST(IDX)%CUNITS     = 'm'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Total ACcumulated PRecipitation rate'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
!No permanent variable associated to this field
!ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VT_FLX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VT_FLX'
TFIELDLIST(IDX)%CUNITS     = 'K m s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'WT_FLX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'WT_FLX'
TFIELDLIST(IDX)%CUNITS     = 'K m s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 4
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RTHS_EDDY_FLUX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RTHS_EDDY_FLUX'
!TODO PW: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VU_FLX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VU_FLX'
TFIELDLIST(IDX)%CUNITS     = 'm s-2'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'RVS_EDDY_FLUX'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'RVS_EDDY_FLUX'
!TODO PW: units?
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .TRUE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'FRC'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'FRC'
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = '--'
TFIELDLIST(IDX)%CCOMMENT   = 'Number of forcing profiles'
TFIELDLIST(IDX)%NGRID      = 0
TFIELDLIST(IDX)%NTYPE      = TYPEINT
TFIELDLIST(IDX)%NDIMS      = 0
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
IDX = IDX+1
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
IF (TRIM(CPROGRAM)=='REAL' .OR. TRIM(CPROGRAM) == 'LFICDF') THEN
!PW: not yet known: IF (LFILTERING) THEN
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UT15'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'UT15'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of Total wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VT15'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VT15'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of Total wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TEMPTOT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TEMPTOT'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_TOTal TEMPerature'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRESTOT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRESTOT'
TFIELDLIST(IDX)%CUNITS     = 'Pa'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_TOTal PRESsure'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'HUMTOT'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'HUMTOT'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_TOTal specific HUMidity'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UT16'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'UT16'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of Environmental wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VT16'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VT16'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of Environmental wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TEMPENV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TEMPENV'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_ENVironmental TEMPerature'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRESENV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRESENV'
TFIELDLIST(IDX)%CUNITS     = 'Pa'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_ENVironmental PRESsure'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 2
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X2D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'HUMENV'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'HUMENV'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_ENVironmental specific HUMidity'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'UT17'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'UT17'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_U component of Basic wind'
TFIELDLIST(IDX)%NGRID      = 2
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VT17'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VT17'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_V component of Basic wind'
TFIELDLIST(IDX)%NGRID      = 3
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'TEMPBAS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'TEMPBAS'
TFIELDLIST(IDX)%CUNITS     = 'K'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_BASic TEMPerature'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'PRESBAS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'PRESBAS'
TFIELDLIST(IDX)%CUNITS     = 'Pa'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_BASic PRESsure'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'HUMBAS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'HUMBAS'
TFIELDLIST(IDX)%CUNITS     = 'kg kg-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_BASic specific HUMidity'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = 'VTDIS'
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = 'VTDIS'
TFIELDLIST(IDX)%CUNITS     = 'm s-1'
TFIELDLIST(IDX)%CDIR       = 'XY'
TFIELDLIST(IDX)%CCOMMENT   = 'X_Y_Z_Total disturbance tangential wind'
TFIELDLIST(IDX)%NGRID      = 1
TFIELDLIST(IDX)%NTYPE      = TYPEREAL
TFIELDLIST(IDX)%NDIMS      = 3
TFIELDLIST(IDX)%LTIMEDEP   = .FALSE.
ALLOCATE(TFIELDLIST(IDX)%TFIELD_X3D(IMODEL))
IDX = IDX+1
!
!END IF !LFILTERING
END IF !CPROGRAM==REAL .OR. LFICDF
!
!
WRITE(YMSG,'("number of used fields=",I4," out of ",I4)') IDX-1,MAXFIELDS
CALL PRINT_MSG(NVERB_INFO,'GEN','INI_FIELD_LIST',TRIM(YMSG))
!
#if 0
!
IF(IDX>MAXFIELDS) CALL ERR_INI_FIELD_LIST()
TFIELDLIST(IDX)%CMNHNAME   = ''
TFIELDLIST(IDX)%CSTDNAME   = ''
TFIELDLIST(IDX)%CLONGNAME  = ''
TFIELDLIST(IDX)%CUNITS     = ''
TFIELDLIST(IDX)%CDIR       = ''
TFIELDLIST(IDX)%CLBTYPE    = ''
TFIELDLIST(IDX)%CCOMMENT   = ''
TFIELDLIST(IDX)%NGRID      = 
TFIELDLIST(IDX)%NTYPE      = 
TFIELDLIST(IDX)%NDIMS      = 
TFIELDLIST(IDX)%LTIMEDEP   = 
ALLOCATE(TFIELDLIST(IDX)%TFIELD_xxxD(IMODEL))
IDX = IDX+1
#endif
!
CONTAINS
SUBROUTINE ERR_INI_FIELD_LIST()
  WRITE(YMSG,'( "IDX>MAXFIELDS (",I5,")" )') MAXFIELDS
  CALL PRINT_MSG(NVERB_FATAL,'GEN','INI_FIELD_LIST',TRIM(YMSG))
END SUBROUTINE ERR_INI_FIELD_LIST
!
END SUBROUTINE INI_FIELD_LIST
!
SUBROUTINE FIND_FIELD_ID_FROM_MNHNAME(HMNHNAME,KID,KRESP,ONOWARNING)
!
CHARACTER(LEN=*),            INTENT(IN) :: HMNHNAME !Name of the field to find
INTEGER,                     INTENT(OUT):: KID      !Index of the field
INTEGER,                     INTENT(OUT):: KRESP    !Return-code 
LOGICAL, OPTIONAL,           INTENT(IN) :: ONOWARNING !If true, do not print warning
!
INTEGER :: IDX,JI
INTEGER :: ICOUNT
INTEGER,SAVE :: IFIRSTGUESS=1 !Store first field to test
CHARACTER(LEN=64) :: YMSG
LOGICAL :: GNOWARNING
!
!PW: TODO: possible optimizations:
! * Classement alphanumerique + index vers 1er champ commencant par caractere
! * Classement dans l'ordre des ecritures + stockage dernier hit + reboucler depuis le debut => DONE
!
IF (.NOT.LFIELDLIST_ISINIT) THEN
  CALL PRINT_MSG(NVERB_FATAL,'GEN','FIND_FIELD_ID_FROM_MNHNAME','TFIELDLIST not yet initialized')
END IF
!
KID = 0
KRESP = 0
ICOUNT = 0
IDX = IFIRSTGUESS
!
IF (PRESENT(ONOWARNING)) THEN
  GNOWARNING = ONOWARNING
ELSE
  GNOWARNING = .FALSE.
END IF
!
DO
  ICOUNT = ICOUNT + 1
  IF (TRIM(TFIELDLIST(IDX)%CMNHNAME)=='') THEN !Last entry
    IDX = 1
  ELSE IF (TRIM(TFIELDLIST(IDX)%CMNHNAME)==TRIM(HMNHNAME)) THEN
    KID = IDX
    EXIT
  ELSE 
    IDX = IDX + 1
    IF (IDX>MAXFIELDS) IDX = 1
  END IF
  IF (IDX == IFIRSTGUESS) EXIT !All entries have been tested
END DO
!
IF (KID==0) THEN
  !Field not found
  KRESP = -1
  IF (.NOT.GNOWARNING) THEN
    CALL PRINT_MSG(NVERB_WARNING,'GEN','FIND_FIELD_ID_FROM_MNHNAME','field '//TRIM(HMNHNAME)//' not known')
  ELSE
    CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIND_FIELD_ID_FROM_MNHNAME','field '//TRIM(HMNHNAME)//' not known (not unexpected)')
  END IF
ELSE
  IFIRSTGUESS = IDX+1
  IF (IFIRSTGUESS>MAXFIELDS) IFIRSTGUESS = 1
  WRITE(YMSG,'( "field ",A16," found after ",I4," attempt(s)" )') TRIM(HMNHNAME),ICOUNT
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIND_FIELD_ID_FROM_MNHNAME',TRIM(YMSG))
END IF
!
END SUBROUTINE FIND_FIELD_ID_FROM_MNHNAME
!
!
SUBROUTINE ALLOC_FIELD_SCALARS
!
USE MODD_DYN_n
USE MODD_PARAM_n
!
CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS','called')
!
IF (LFIELDLIST_ISINIT) THEN
  CALL PRINT_MSG(NVERB_FATAL,'GEN','ALLOC_FIELD_SCALARS','TFIELDLIST already initialized')
END IF
!
!
IF (.NOT.ASSOCIATED(NRIMX)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS',' NRIMX was not associated')
  ALLOCATE(NRIMX)
END IF
IF (.NOT.ASSOCIATED(NRIMY)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS',' NRIMY was not associated')
  ALLOCATE(NRIMY)
END IF
IF (.NOT.ASSOCIATED(LHORELAX_UVWTH)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS',' LHORELAX_UVWTH was not associated')
  ALLOCATE(LHORELAX_UVWTH)
END IF
IF (.NOT.ASSOCIATED(LHORELAX_TKE)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS',' LHORELAX_TKE was not associated')
  ALLOCATE(LHORELAX_TKE)
END IF
IF (.NOT.ASSOCIATED(CSURF)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','ALLOC_FIELD_SCALARS',' CSURF was not associated')
  ALLOCATE(CHARACTER(LEN=4) :: CSURF)
  CSURF = ''
END IF
!
END SUBROUTINE ALLOC_FIELD_SCALARS
!
!
SUBROUTINE INI_FIELD_SCALARS
!
USE MODD_DYN_n
USE MODD_FIELD_n
USE MODD_GRID_n
USE MODD_TIME_n
USE MODD_PARAM_n
!
INTEGER :: IID,IRESP
!
CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS','called')
!
IF (.NOT.LFIELDLIST_ISINIT) THEN
  CALL PRINT_MSG(NVERB_FATAL,'GEN','FIND_FIELD_ID_FROM_MNHNAME','TFIELDLIST not yet initialized')
END IF
!
!
IF (.NOT.ASSOCIATED(XZTOP)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' XZTOP was not associated')
  ALLOCATE(XZTOP)
  CALL FIND_FIELD_ID_FROM_MNHNAME('ZTOP',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_X0D(1)%DATA=>XZTOP
END IF
!
IF (.NOT.ASSOCIATED(LSLEVE)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' LSLEVE was not associated')
  ALLOCATE(LSLEVE)
  CALL FIND_FIELD_ID_FROM_MNHNAME('SLEVE',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_L0D(1)%DATA=>LSLEVE
END IF
!
IF (.NOT.ASSOCIATED(XLEN1)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' XLEN1 was not associated')
  ALLOCATE(XLEN1)
  CALL FIND_FIELD_ID_FROM_MNHNAME('LEN1',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_X0D(1)%DATA=>XLEN1
END IF
!
IF (.NOT.ASSOCIATED(XLEN2)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' XLEN2 was not associated')
  ALLOCATE(XLEN2)
  CALL FIND_FIELD_ID_FROM_MNHNAME('LEN2',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_X0D(1)%DATA=>XLEN2
END IF
!
IF (.NOT.ASSOCIATED(TDTMOD)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' TDTMOD was not associated')
  ALLOCATE(TDTMOD)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTMOD',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_T0D(1)%DATA=>TDTMOD
END IF
!
IF (.NOT.ASSOCIATED(TDTCUR)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' TDTCUR was not associated')
  ALLOCATE(TDTCUR)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTCUR',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_T0D(1)%DATA=>TDTCUR
END IF
!
IF (.NOT.ASSOCIATED(TDTRAD_FULL)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' TDTRAD_FULL was not associated')
  ALLOCATE(TDTRAD_FULL)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTRAD_FULL',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_T0D(1)%DATA=>TDTRAD_FULL
END IF
!
IF (.NOT.ASSOCIATED(TDTRAD_CLONLY)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' TDTRAD_CLONLY was not associated')
  ALLOCATE(TDTRAD_CLONLY)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTRAD_CLLY',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_T0D(1)%DATA=>TDTRAD_CLONLY
END IF
!
IF (.NOT.ASSOCIATED(TDTDCONV)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' TDTDCONV was not associated')
  ALLOCATE(TDTDCONV)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTDCONV',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_T0D(1)%DATA=>TDTDCONV
END IF
!
IF (.NOT.ASSOCIATED(CSURF)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' CSURF was not associated')
  ALLOCATE(CHARACTER(LEN=4) :: CSURF)
  CSURF = ''
END IF
CALL FIND_FIELD_ID_FROM_MNHNAME('SURF',IID,IRESP)
TFIELDLIST(IID)%TFIELD_C0D(1)%DATA=>CSURF
!
IF (.NOT.ASSOCIATED(XDRYMASST)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' XDRYMASST was not associated')
  ALLOCATE(XDRYMASST)
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRYMASST',IID,IRESP)
  TFIELDLIST(IID)%TFIELD_X0D(1)%DATA=>XDRYMASST
END IF
!
IF (.NOT.ASSOCIATED(NRIMX)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' NRIMX was not associated')
  ALLOCATE(NRIMX)
END IF
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RIMX',IID,IRESP)
TFIELDLIST(IID)%TFIELD_N0D(1)%DATA=>NRIMX
IF (.NOT.ASSOCIATED(NRIMY)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' NRIMY was not associated')
  ALLOCATE(NRIMY)
END IF
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RIMY',IID,IRESP)
TFIELDLIST(IID)%TFIELD_N0D(1)%DATA=>NRIMY
!
IF (.NOT.ASSOCIATED(LHORELAX_UVWTH)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' LHORELAX_UVWTH was not associated')
  ALLOCATE(LHORELAX_UVWTH)
END IF
CALL FIND_FIELD_ID_FROM_MNHNAME('HORELAX_UVWTH',IID,IRESP)
TFIELDLIST(IID)%TFIELD_L0D(1)%DATA=>LHORELAX_UVWTH
!
IF (.NOT.ASSOCIATED(LHORELAX_TKE)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','INI_FIELD_SCALARS',' LHORELAX_TKE was not associated')
  ALLOCATE(LHORELAX_TKE)
END IF
CALL FIND_FIELD_ID_FROM_MNHNAME('HORELAX_TKE',IID,IRESP)
TFIELDLIST(IID)%TFIELD_L0D(1)%DATA=>LHORELAX_TKE
!
END SUBROUTINE INI_FIELD_SCALARS
!
!
SUBROUTINE FIELDLIST_GOTO_MODEL(KFROM, KTO)
!
USE MODD_REF
!
USE MODD_ADV_n
USE MODD_CONF_n
USE MODD_DEEP_CONVECTION_n
USE MODD_DEF_EDDY_FLUX_n
USE MODD_DEF_EDDYUV_FLUX_n
USE MODD_DYN_n
USE MODD_ELEC_n
USE MODD_FIELD_n
USE MODD_GR_FIELD_n
USE MODD_GRID_n
USE MODD_HURR_FIELD_n
USE MODD_LIMA_PRECIP_SCAVENGING_n
USE MODD_LSFIELD_n
USE MODD_PARAM_n
USE MODD_PAST_FIELD_n
USE MODD_CH_PH_n
USE MODD_PRECIP_n
USE MODD_RADIATIONS_n
USE MODD_REF_n
USE MODD_TIME_n
USE MODD_TURB_n
!
INTEGER, INTENT(IN) :: KFROM, KTO
!
!LOGICAL,SAVE :: GFIRST_CALL=.TRUE.
INTEGER :: IID,IID2,IRESP
CHARACTER(LEN=64) :: YMSG
!
WRITE(YMSG,'( I4,"->",I4 )') KFROM,KTO
CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',TRIM(YMSG))
!
! IF (GFIRST_CALL) THEN
!   !This is necessary because the first time this subroutine is called
!   !the TFIELDLIST is not yet initialized.
!   !The use of this subroutine is not useful the first timebecause the
!   !data for the fields has not yet been allocated.
!   GFIRST_CALL = .FALSE.
!   RETURN
! END IF
!
IF (.NOT.LFIELDLIST_ISINIT) THEN
  CALL PRINT_MSG(NVERB_WARNING,'GEN','FIELDLIST_GOTO_MODEL','TFIELDLIST not yet initialized')
  RETURN
END IF
!
! Initialize some pointers
!
!PW: TODO: check if still necessary as XRHODREFZ and XTHVREFZ are now initialiazed in ini_modeln even for KMI/=1 (29/01/2019)
IF (KFROM == KTO) THEN
  IF ( CPROGRAM/='NESPGD' .AND. CPROGRAM/='PGD' ) THEN
    IF (.NOT.ALLOCATED(XRHODREFZ)) CALL PRINT_MSG(NVERB_FATAL,'GEN','FIELDLIST_GOTO_MODEL','XRHODREFZ not yet allocated')
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHOREFZ',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA=>XRHODREFZ
    !
    IF (.NOT.ALLOCATED(XTHVREFZ)) CALL PRINT_MSG(NVERB_FATAL,'GEN','FIELDLIST_GOTO_MODEL','XTHVREFZ not yet allocated')
    CALL FIND_FIELD_ID_FROM_MNHNAME('THVREFZ',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA=>XTHVREFZ
  END IF
END IF
!
!
!
!
! Save current state for allocated arrays
!
!
!
!
!
! MODD_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('UT',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUT
CALL FIND_FIELD_ID_FROM_MNHNAME('VT',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVT
CALL FIND_FIELD_ID_FROM_MNHNAME('WT',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XWT
CALL FIND_FIELD_ID_FROM_MNHNAME('THT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTHT
CALL FIND_FIELD_ID_FROM_MNHNAME('TKET', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTKET
CALL FIND_FIELD_ID_FROM_MNHNAME('PABST',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPABST
CALL FIND_FIELD_ID_FROM_MNHNAME('RT',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X4D(KFROM)%DATA => XRT
!
IF (ASSOCIATED(XRT)) THEN
  IF (CONF_MODEL(KFROM)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RVT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RCT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RRT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RIT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RST',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RST)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RGT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>XRT(:,:,:,CONF_MODEL(KFROM)%IDX_RHT)
  END IF
ELSE
  IF (CONF_MODEL(KFROM)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RST',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA=>NULL()
  END IF

END IF
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SUPSATMAX',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSUPSAT
CALL FIND_FIELD_ID_FROM_MNHNAME('NACT',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XNACT
CALL FIND_FIELD_ID_FROM_MNHNAME('SSPRO',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSSPRO
CALL FIND_FIELD_ID_FROM_MNHNAME('NPRO',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XNPRO
CALL FIND_FIELD_ID_FROM_MNHNAME('SRCT',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSRCT
CALL FIND_FIELD_ID_FROM_MNHNAME('SIGS',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSIGS
!
IF (CPROGRAM == 'MESONH') THEN
  !
  CALL FIND_FIELD_ID_FROM_MNHNAME('US_PRES',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRUS_PRES
  CALL FIND_FIELD_ID_FROM_MNHNAME('VS_PRES',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRVS_PRES
  CALL FIND_FIELD_ID_FROM_MNHNAME('WS_PRES',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRWS_PRES
  CALL FIND_FIELD_ID_FROM_MNHNAME('THS_CLD',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRTHS_CLD
  !
  CALL FIND_FIELD_ID_FROM_MNHNAME('RS_CLD',IID,IRESP); TFIELDLIST(IID)%TFIELD_X4D(KFROM)%DATA => XRRS_CLD
  !
  IF (CONF_MODEL(KFROM)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RVT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RCT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RRT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RIT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RSS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RST)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RGT)
  END IF
  IF (CONF_MODEL(KFROM)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHS_CLD',IID,IRESP)
    TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRRS_CLD(:,:,:,CONF_MODEL(KFROM)%IDX_RHT)
  END IF
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLDFR',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XCLDFR
  CALL FIND_FIELD_ID_FROM_MNHNAME('CIT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XCIT
  !
END IF
!
! MODD_PAST_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('UM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUM
CALL FIND_FIELD_ID_FROM_MNHNAME('VM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVM
CALL FIND_FIELD_ID_FROM_MNHNAME('WM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XWM
CALL FIND_FIELD_ID_FROM_MNHNAME('DUM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDUM
CALL FIND_FIELD_ID_FROM_MNHNAME('DVM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDVM
CALL FIND_FIELD_ID_FROM_MNHNAME('DWM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDWM
!
! MODD_LIMA_PRECIP_SCAVENGING_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('INPAP',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPAP
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPAP',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPAP
!
! MODD_ELEC_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDU',IID,IRESP);  TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XEFIELDU
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDV',IID,IRESP);  TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XEFIELDV
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDW',IID,IRESP);  TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XEFIELDW
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_IAGGS',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XNI_IAGGS
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_IDRYG',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XNI_IDRYG
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_SDRYG',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XNI_SDRYG
CALL FIND_FIELD_ID_FROM_MNHNAME('INDUC_CG',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XIND_RATE
!
! MODD_CH_PH_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('PHC',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPHC
CALL FIND_FIELD_ID_FROM_MNHNAME('PHR',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPHR
!
! MODD_LSFIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('LSUM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLSUM
CALL FIND_FIELD_ID_FROM_MNHNAME('LSVM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLSVM
CALL FIND_FIELD_ID_FROM_MNHNAME('LSWM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLSWM
CALL FIND_FIELD_ID_FROM_MNHNAME('LSTHM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLSTHM
IF(LUSERV) CALL FIND_FIELD_ID_FROM_MNHNAME('LSRVM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLSRVM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXUM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBXUM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXVM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBXVM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXWM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBXWM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYUM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBYUM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYVM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBYVM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYWM', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBYWM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXTHM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBXTHM
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYTHM',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XLBYTHM
!
! MODD_DYN_n variables
!
!***NONE***
!
! MODD_ADV_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('TKEMS',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRTKEMS
!
! MODD_GRID_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('ZS',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XZS
CALL FIND_FIELD_ID_FROM_MNHNAME('ZSMT', IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XZSMT
CALL FIND_FIELD_ID_FROM_MNHNAME('XHAT', IID,IRESP); TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA => XXHAT
CALL FIND_FIELD_ID_FROM_MNHNAME('YHAT', IID,IRESP); TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA => XYHAT
CALL FIND_FIELD_ID_FROM_MNHNAME('ZHAT', IID,IRESP); TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA => XZHAT
CALL FIND_FIELD_ID_FROM_MNHNAME('ZTOP', IID,IRESP); TFIELDLIST(IID)%TFIELD_X0D(KFROM)%DATA => XZTOP
CALL FIND_FIELD_ID_FROM_MNHNAME('DXHAT',IID,IRESP); TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA => XDXHAT
CALL FIND_FIELD_ID_FROM_MNHNAME('DYHAT',IID,IRESP); TFIELDLIST(IID)%TFIELD_X1D(KFROM)%DATA => XDYHAT
CALL FIND_FIELD_ID_FROM_MNHNAME('ALT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XZZ
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSXW',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XDIRCOSXW
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSYW',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XDIRCOSYW
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSZW',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XDIRCOSZW
CALL FIND_FIELD_ID_FROM_MNHNAME('COSSLOPE',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XCOSSLOPE
CALL FIND_FIELD_ID_FROM_MNHNAME('SINSLOPE',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSINSLOPE
CALL FIND_FIELD_ID_FROM_MNHNAME('MAP',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XMAP
CALL FIND_FIELD_ID_FROM_MNHNAME('LAT',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XLAT
CALL FIND_FIELD_ID_FROM_MNHNAME('LON',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XLON
!
! MODD_TIME_n variables
!
!***NONE***
!
! MODD_PARAM_n variables
!
!***NONE***
!
! MODD_TURB_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('BL_DEPTH', IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XBL_DEPTH
CALL FIND_FIELD_ID_FROM_MNHNAME('SBL_DEPTH',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSBL_DEPTH
CALL FIND_FIELD_ID_FROM_MNHNAME('WTHVMF',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XWTHVMF
!
! MODD_REF_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RHODREF',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRHODREF
CALL FIND_FIELD_ID_FROM_MNHNAME('THVREF', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTHVREF
!
! MODD_RADIATIONS_n variables
!
IF (CPROGRAM=='MESONH') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTHRAD',      IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDTHRAD
  CALL FIND_FIELD_ID_FROM_MNHNAME('FLALWD',      IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XFLALWD
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIRFLASWD',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDIRFLASWD
  CALL FIND_FIELD_ID_FROM_MNHNAME('SCAFLASWD',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSCAFLASWD
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIRSRFSWD',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDIRSRFSWD
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLEARCOL_TM1',IID,IRESP); TFIELDLIST(IID)%TFIELD_N2D(KFROM)%DATA => NCLEARCOL_TM1
  CALL FIND_FIELD_ID_FROM_MNHNAME('ZENITH',      IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XZENITH
  CALL FIND_FIELD_ID_FROM_MNHNAME('AZIM',        IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XAZIM
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIR_ALB',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDIR_ALB
  CALL FIND_FIELD_ID_FROM_MNHNAME('SCA_ALB',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XSCA_ALB
  CALL FIND_FIELD_ID_FROM_MNHNAME('EMIS',        IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XEMIS
  CALL FIND_FIELD_ID_FROM_MNHNAME('TSRAD',       IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XTSRAD
END IF
!
! MODD_DEEP_CONVECTION_n variables
!
IF (TRIM(CPROGRAM) /= 'PGD' .AND. TRIM(CPROGRAM) /= 'NESPGD' .AND. TRIM(CPROGRAM) /= 'SPAWN') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('COUNTCONV',  IID,IRESP); TFIELDLIST(IID)%TFIELD_N2D(KFROM)%DATA => NCOUNTCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTHCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDTHCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRVCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDRVCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRCCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDRCCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRICONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDRICONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRCONV',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XPRCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PACCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XPACCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRSCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XPRSCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DSVCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X4D(KFROM)%DATA => XDSVCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRLFLXCONV', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPRLFLXCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRSFLXCONV', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPRSFLXCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('UMFCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUMFCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('DMFCONV',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XDMFCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('MFCONV',     IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XMFCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('CAPE',       IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XCAPE
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLTOPCONV_LVL',IID,IRESP); TFIELDLIST(IID)%TFIELD_N2D(KFROM)%DATA => NCLTOPCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLBASCONV_LVL',IID,IRESP); TFIELDLIST(IID)%TFIELD_N2D(KFROM)%DATA => NCLBASCONV
  CALL FIND_FIELD_ID_FROM_MNHNAME('IC_RATE',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XIC_RATE
  CALL FIND_FIELD_ID_FROM_MNHNAME('CG_RATE',    IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XCG_RATE
  CALL FIND_FIELD_ID_FROM_MNHNAME('IC_TOTAL_NB',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XIC_TOTAL_NUMBER
  CALL FIND_FIELD_ID_FROM_MNHNAME('CG_TOTAL_NB',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XCG_TOTAL_NUMBER
END IF
!
! MODD_GR_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_ANIS', IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSSO_ANISOTROPY
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_SLOPE',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSSO_SLOPE
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_DIR',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSSO_DIRECTION
CALL FIND_FIELD_ID_FROM_MNHNAME('AVG_ZS',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XAVG_ZS
CALL FIND_FIELD_ID_FROM_MNHNAME('SIL_ZS',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSIL_ZS
CALL FIND_FIELD_ID_FROM_MNHNAME('MAX_ZS',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XMAX_ZS
CALL FIND_FIELD_ID_FROM_MNHNAME('MIN_ZS',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XMIN_ZS
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_STDEV',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XSSO_STDEV
!
! MODD_PRECIP_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRC',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPRC
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRC',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPRC
CALL FIND_FIELD_ID_FROM_MNHNAME('INDEP',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINDEP
CALL FIND_FIELD_ID_FROM_MNHNAME('ACDEP',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACDEP
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRR',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPRR
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRR3D',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XINPRR3D
CALL FIND_FIELD_ID_FROM_MNHNAME('EVAP3D', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XEVAP3D
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRR',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPRR
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRS',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPRS
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRS',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPRS
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRG',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPRG
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRG',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPRG
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRH',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XINPRH
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRH',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XACPRH
!
! MODD_DEF_EDDY_FLUX_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('VT_FLX',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA         => XVTH_FLUX_M
CALL FIND_FIELD_ID_FROM_MNHNAME('WT_FLX',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA         => XWTH_FLUX_M
CALL FIND_FIELD_ID_FROM_MNHNAME('RTHS_EDDY_FLUX',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRTHS_EDDY_FLUX
!
! MODD_DEF_EDDYUV_FLUX_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('VU_FLX',IID,IRESP);        TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVU_FLUX_M
CALL FIND_FIELD_ID_FROM_MNHNAME('RVS_EDDY_FLUX',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XRVS_EDDY_FLUX
!
! MODD_HURR_FIELD_n variables
!
IF (CPROGRAM=='REAL') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT15',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUTOT
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT15',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVTOT
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPTOT',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTTOT
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESTOT',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XPTOT
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMTOT', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XQTOT
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT16',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUENV
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT16',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVENV
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPENV',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTENV
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESENV',IID,IRESP); TFIELDLIST(IID)%TFIELD_X2D(KFROM)%DATA => XPENV
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMENV', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XQENV
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT17',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XUBASIC
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT17',   IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVBASIC
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPBAS',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XTBASIC
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESBAS',IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XPBASIC
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMBAS', IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XQBASIC
  CALL FIND_FIELD_ID_FROM_MNHNAME('VTDIS',  IID,IRESP); TFIELDLIST(IID)%TFIELD_X3D(KFROM)%DATA => XVTDIS
END IF
!
!
!
!
! Current model is set to model KTO
!
!
!
!
IF( KFROM/=KTO) THEN
!
! MODD_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('UT',   IID,IRESP); XUT    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('VT',   IID,IRESP); XVT    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('WT',   IID,IRESP); XWT    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('THT',  IID,IRESP); XTHT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('TKET', IID,IRESP); XTKET  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('PABST',IID,IRESP); XPABST => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('RT',   IID,IRESP); XRT    => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA
!
IF (ASSOCIATED(XRT)) THEN
  IF (CONF_MODEL(KTO)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RVT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RCT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RRT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RIT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RST',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RST)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RGT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RHT)
  END IF
ELSE
  IF (CONF_MODEL(KTO)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RST',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
  IF (CONF_MODEL(KTO)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHT',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => NULL()
  END IF
END IF
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SUPSATMAX',IID,IRESP); XSUPSAT => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('NACT',     IID,IRESP); XNACT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SSPRO',    IID,IRESP); XSSPRO  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('NPRO',     IID,IRESP); XNPRO   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SRCT',     IID,IRESP); XSRCT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SIGS',     IID,IRESP); XSIGS   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
IF (CPROGRAM == 'MESONH') THEN
  !
  CALL FIND_FIELD_ID_FROM_MNHNAME('RS_CLD',IID,IRESP); XRRS_CLD => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA
  !
  IF (CONF_MODEL(KTO)%IDX_RVT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RVS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RVT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RCT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RCS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RCT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RRT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RRS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RRT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RIT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RIS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RIT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RST>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RSS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RST)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RGT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RGS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RGT)
  END IF
  IF (CONF_MODEL(KTO)%IDX_RHT>0) THEN
    CALL FIND_FIELD_ID_FROM_MNHNAME('RHS_CLD',IID2,IRESP)
    TFIELDLIST(IID2)%TFIELD_X3D(KTO)%DATA => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA(:,:,:,CONF_MODEL(KTO)%IDX_RHT)
  END IF
  !
  CALL FIND_FIELD_ID_FROM_MNHNAME('US_PRES',IID,IRESP); XRUS_PRES => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('VS_PRES',IID,IRESP); XRVS_PRES => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('WS_PRES',IID,IRESP); XRWS_PRES => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('THS_CLD',IID,IRESP); XRTHS_CLD => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLDFR',  IID,IRESP); XCLDFR    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CIT',    IID,IRESP); XCIT      => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
END IF
!
! MODD_PAST_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('UM', IID,IRESP); XUM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('VM', IID,IRESP); XVM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('WM', IID,IRESP); XWM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DUM',IID,IRESP); XDUM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DVM',IID,IRESP); XDVM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DWM',IID,IRESP); XDWM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_LIMA_PRECIP_SCAVENGING_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('INPAP',IID,IRESP); XINPAP => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPAP',IID,IRESP); XACPAP => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
!
! MODD_ELEC_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDU',IID,IRESP);  XEFIELDU  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDV',IID,IRESP);  XEFIELDV  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('EFIELDW',IID,IRESP);  XEFIELDW  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_IAGGS',IID,IRESP); XNI_IAGGS => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_IDRYG',IID,IRESP); XNI_IDRYG => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('NI_SDRYG',IID,IRESP); XNI_SDRYG => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INDUC_CG',IID,IRESP); XIND_RATE => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_CH_PH_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('PHC',IID,IRESP); XPHC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('PHR',IID,IRESP); XPHR => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_LSFIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('LSUM', IID,IRESP); XLSUM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LSVM', IID,IRESP); XLSVM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LSWM', IID,IRESP); XLSWM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LSTHM',IID,IRESP); XLSTHM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
IF(LUSERV) CALL FIND_FIELD_ID_FROM_MNHNAME('LSRVM',IID,IRESP); XLSRVM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXUM', IID,IRESP); XLBXUM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXVM', IID,IRESP); XLBXVM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXWM', IID,IRESP); XLBXWM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYUM', IID,IRESP); XLBYUM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYVM', IID,IRESP); XLBYVM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYWM', IID,IRESP); XLBYWM  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBXTHM',IID,IRESP); XLBXTHM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LBYTHM',IID,IRESP); XLBYTHM => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DRYMASST',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)
END IF
XDRYMASST => TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA
!
! MODD_DYN_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RIMX',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA)
END IF
NRIMX => TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RIMY',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA)
END IF
NRIMY => TFIELDLIST(IID)%TFIELD_N0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('HORELAX_UVWTH',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)
END IF
LHORELAX_UVWTH => TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('HORELAX_TKE',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)
END IF
LHORELAX_TKE => TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA
!
! MODD_ADV_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('TKEMS',IID,IRESP); XRTKEMS=>TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_GRID_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('ZS',   IID,IRESP); XZS    => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ZSMT', IID,IRESP); XZSMT  => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('XHAT', IID,IRESP); XXHAT  => TFIELDLIST(IID)%TFIELD_X1D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('YHAT', IID,IRESP); XYHAT  => TFIELDLIST(IID)%TFIELD_X1D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ZHAT', IID,IRESP); XZHAT  => TFIELDLIST(IID)%TFIELD_X1D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('ZTOP', IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)
END IF
XZTOP  => TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DXHAT',IID,IRESP); XDXHAT => TFIELDLIST(IID)%TFIELD_X1D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DYHAT',IID,IRESP); XDYHAT => TFIELDLIST(IID)%TFIELD_X1D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SLEVE',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA)
END IF
LSLEVE => TFIELDLIST(IID)%TFIELD_L0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('LEN1',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)
END IF
XLEN1 => TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('LEN2',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA)
END IF
XLEN2 => TFIELDLIST(IID)%TFIELD_X0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('ALT',     IID,IRESP); XZZ       => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSXW',IID,IRESP); XDIRCOSXW => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSYW',IID,IRESP); XDIRCOSYW => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('DIRCOSZW',IID,IRESP); XDIRCOSZW => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('COSSLOPE',IID,IRESP); XCOSSLOPE => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SINSLOPE',IID,IRESP); XSINSLOPE => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('MAP',  IID,IRESP); XMAP => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LAT',  IID,IRESP); XLAT => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('LON',  IID,IRESP); XLON => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
!
! MODD_TIME_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DTMOD',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)
END IF
TDTMOD => TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DTCUR',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)
END IF
TDTCUR => TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DTRAD_FULL',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)
END IF
TDTRAD_FULL => TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DTRAD_CLLY',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)
END IF
TDTRAD_CLONLY => TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA
!
CALL FIND_FIELD_ID_FROM_MNHNAME('DTDCONV',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA)
END IF
TDTDCONV => TFIELDLIST(IID)%TFIELD_T0D(KTO)%DATA
!
! MODD_PARAM_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SURF',IID,IRESP)
IF (.NOT.ASSOCIATED(TFIELDLIST(IID)%TFIELD_C0D(KTO)%DATA)) THEN
  CALL PRINT_MSG(NVERB_DEBUG,'GEN','FIELDLIST_GOTO_MODEL',&
                 'TFIELDLIST(IID)%TFIELD_C0D(KTO)%DATA was not associated for '//TFIELDLIST(IID)%CMNHNAME)
  ALLOCATE(CHARACTER(LEN=4) :: TFIELDLIST(IID)%TFIELD_C0D(KTO)%DATA)
END IF
CSURF => TFIELDLIST(IID)%TFIELD_C0D(KTO)%DATA
!
! MODD_TURB_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('BL_DEPTH', IID,IRESP); XBL_DEPTH  => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SBL_DEPTH',IID,IRESP); XSBL_DEPTH => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('WTHVMF',   IID,IRESP); XWTHVMF    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_REF_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('RHODREF',IID,IRESP); XRHODREF => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('THVREF', IID,IRESP); XTHVREF  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_RADIATIONS_n variables
!
IF (CPROGRAM=='MESONH') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTHRAD',      IID,IRESP); XDTHRAD       => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('FLALWD',      IID,IRESP); XFLALWD       => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIRFLASWD',   IID,IRESP); XDIRFLASWD    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('SCAFLASWD',   IID,IRESP); XSCAFLASWD    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIRSRFSWD',   IID,IRESP); XDIRSRFSWD    => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLEARCOL_TM1',IID,IRESP); NCLEARCOL_TM1 => TFIELDLIST(IID)%TFIELD_N2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('ZENITH',      IID,IRESP); XZENITH       => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('AZIM',        IID,IRESP); XAZIM         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DIR_ALB',     IID,IRESP); XDIR_ALB      => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('SCA_ALB',     IID,IRESP); XSCA_ALB      => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('EMIS',        IID,IRESP); XEMIS         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('TSRAD',       IID,IRESP); XTSRAD        => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
END IF
!
! MODD_DEEP_CONVECTION_n variables
!
IF (TRIM(CPROGRAM) /= 'PGD' .AND. TRIM(CPROGRAM) /= 'NESPGD' .AND. TRIM(CPROGRAM) /= 'SPAWN') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('COUNTCONV',  IID,IRESP); NCOUNTCONV       => TFIELDLIST(IID)%TFIELD_N2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DTHCONV',    IID,IRESP); XDTHCONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRVCONV',    IID,IRESP); XDRVCONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRCCONV',    IID,IRESP); XDRCCONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DRICONV',    IID,IRESP); XDRICONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRCONV',     IID,IRESP); XPRCONV          => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PACCONV',    IID,IRESP); XPACCONV         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRSCONV',    IID,IRESP); XPRSCONV         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DSVCONV',    IID,IRESP); XDSVCONV         => TFIELDLIST(IID)%TFIELD_X4D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRLFLXCONV', IID,IRESP); XPRLFLXCONV      => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRSFLXCONV', IID,IRESP); XPRSFLXCONV      => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('UMFCONV',    IID,IRESP); XUMFCONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('DMFCONV',    IID,IRESP); XDMFCONV         => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('MFCONV',     IID,IRESP); XMFCONV          => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CAPE',       IID,IRESP); XCAPE            => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLTOPCONV_LVL',IID,IRESP); NCLTOPCONV     => TFIELDLIST(IID)%TFIELD_N2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CLBASCONV_LVL',IID,IRESP); NCLBASCONV     => TFIELDLIST(IID)%TFIELD_N2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('IC_RATE',    IID,IRESP); XIC_RATE         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CG_RATE',    IID,IRESP); XCG_RATE         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('IC_TOTAL_NB',IID,IRESP); XIC_TOTAL_NUMBER => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('CG_TOTAL_NB',IID,IRESP); XCG_TOTAL_NUMBER => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
END IF
!
! MODD_GR_FIELD_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_ANIS', IID,IRESP); XSSO_ANISOTROPY => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_SLOPE',IID,IRESP); XSSO_SLOPE      => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_DIR',  IID,IRESP); XSSO_DIRECTION  => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('AVG_ZS',   IID,IRESP); XAVG_ZS         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SIL_ZS',   IID,IRESP); XSIL_ZS         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('MAX_ZS',   IID,IRESP); XMAX_ZS         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('MIN_ZS',   IID,IRESP); XMIN_ZS         => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('SSO_STDEV',IID,IRESP); XSSO_STDEV      => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
!
! MODD_PRECIP_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRC',  IID,IRESP); XINPRC   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRC',  IID,IRESP); XACPRC   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INDEP',  IID,IRESP); XINDEP   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACDEP',  IID,IRESP); XACDEP   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRR',  IID,IRESP); XINPRR   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRR3D',IID,IRESP); XINPRR3D => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('EVAP3D', IID,IRESP); XEVAP3D  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRR',  IID,IRESP); XACPRR   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRS',  IID,IRESP); XINPRS   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRS',  IID,IRESP); XACPRS   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRG',  IID,IRESP); XINPRG   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRG',  IID,IRESP); XACPRG   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('INPRH',  IID,IRESP); XINPRH   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('ACPRH',  IID,IRESP); XACPRH   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
!
!
! MODD_DEF_EDDY_FLUX_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('VT_FLX',        IID,IRESP); XVTH_FLUX_M     => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('WT_FLX',        IID,IRESP); XWTH_FLUX_M     => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('RTHS_EDDY_FLUX',IID,IRESP); XRTHS_EDDY_FLUX => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
! MODD_DEF_EDDYUV_FLUX_n variables
!
CALL FIND_FIELD_ID_FROM_MNHNAME('VU_FLX',       IID,IRESP); XVU_FLUX_M     => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
CALL FIND_FIELD_ID_FROM_MNHNAME('RVS_EDDY_FLUX',IID,IRESP); XRVS_EDDY_FLUX => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
!
!
! MODD_HURR_FIELD_n variables
!
IF (CPROGRAM=='REAL') THEN
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT15',   IID,IRESP); XUTOT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT15',   IID,IRESP); XVTOT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPTOT',IID,IRESP); XTTOT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESTOT',IID,IRESP); XPTOT   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMTOT', IID,IRESP); XQTOT   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT16',   IID,IRESP); XUENV   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT16',   IID,IRESP); XVENV   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPENV',IID,IRESP); XTENV   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESENV',IID,IRESP); XPENV   => TFIELDLIST(IID)%TFIELD_X2D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMENV', IID,IRESP); XQENV   => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('UT17',   IID,IRESP); XUBASIC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('VT17',   IID,IRESP); XVBASIC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('TEMPBAS',IID,IRESP); XTBASIC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('PRESBAS',IID,IRESP); XPBASIC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('HUMBAS', IID,IRESP); XQBASIC => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
  CALL FIND_FIELD_ID_FROM_MNHNAME('VTDIS',  IID,IRESP); XVTDIS  => TFIELDLIST(IID)%TFIELD_X3D(KTO)%DATA
END IF
!
END IF !KFROM/=KTO
!
END SUBROUTINE FIELDLIST_GOTO_MODEL
!
END MODULE MODE_FIELD
